@@ -23,16 +23,13 @@ use IO::Handle;
 use strict;
 use warnings;
 
-eval 'use Devel::GlobalDestruction';
-
 our @ISA = qw(Exporter);
 our @EXPORT = qw(@ALL_JOBS %ALL_JOBS PREFORK POSTFORK 
                  POSTFORK_PARENT POSTFORK_CHILD);
-our $VERSION = '0.85';
+our $VERSION = '0.86';
 
 our (@ALL_JOBS, %ALL_JOBS, @ARCHIVED_JOBS, $WIN32_PROC, $WIN32_PROC_PID);
 our $OVERLOAD_ENABLED = 0;
-our $INSIDE_END_QUEUE = 0;
 our $_RETRY_PAUSE;
 our (@PREFORK, @POSTFORK_PARENT, @POSTFORK_CHILD);
 
@@ -391,24 +388,22 @@ sub _can_launch_dependency_check {
 sub _can_launch_remote {
     my $job = shift;
     return 1 if !defined $job->{remote};
-    my @hosts;
-    if (ref($job->{remote}{host}) ne 'ARRAY') {
-        @hosts = $job->{remote}{host};
-    } else {
-        @hosts = @{$job->{remote}{host}};
-    }
-    for (my $i=$#hosts; $i>=1; $i--) {
+    my @specs = @{$job->{remote}};
+    for (my $i=$#specs; $i>=1; $i--) {
         my $j = int(rand($i+1));
-        @hosts[$i,$j] = @hosts[$j,$i];
+        @specs[$i,$j] = @specs[$j,$i];
     }
 
-    foreach my $host (@hosts) {
+    foreach my $spec (@specs) {
+        my $host = $spec->{host};
+        next unless $host;
         if ($job->_can_launch_remote_check_host($host)) {
-            $job->{remote}{host} = $host;
+            $job->{remote} = $spec;
             return 1;
         }
     }
     if ($job->{debug}) {
+        my @hosts = map { $_->{host} } @specs;
         debug("_can_launch_remote: host(s) @hosts are too busy");
     }
     return 0;
@@ -442,6 +437,9 @@ sub max_proc {
     } else {
         $val = $Forks::Super::MAX_PROC;
         my $host = shift;
+        if (ref($host) && ref($host) eq 'Forks::Super::Job') {
+            $host = shift;
+        }
         if (defined($host)) {
             if (defined($Forks::Super::MAX_PROC{$host})) {
                 $val = $Forks::Super::MAX_PROC{$host};
@@ -472,8 +470,6 @@ sub _can_launch {
     no warnings qw(once);
 
     my $job = shift;
-#   my $force = defined($job->{max_load}) && $job->{force};
-
     if ($job->{force}) {
 	debug('_can_launch(): force attr set. launch ok')
 	    if $job->{debug};
@@ -1082,89 +1078,70 @@ sub _postlaunch_child_to_cmd {
     }
 }
 
+#
+# experimental and undocumented 
+#
 sub _postlaunch_child_to_remote_cmd {
     my $job = shift;
     my $remote = $job->{remote};
-    debug("Executing command [ @{$job->{cmd}} ] on "
+    my @cmd = @{$job->{cmd}};
+    debug("Executing command [ @cmd ] on "
           . $remote->{host}) if $job->{debug};
 
+    open my $ZZZTTY,'+<','/dev/tty';
+#    print $ZZZTTY "_postlaunch_child_to_remote_cmd called\n";
+
     my $retries = $job->{retries} || 0;
     my $this_pid = $$;
 
-    if ($remote->{proto} eq 'ssh') {
-        if (!$remote->{password} && CONFIG_module("Net::SSH")) {
+    if (!defined($remote->{proto}) || $remote->{proto} eq 'ssh') {
+        if (CONFIG_module('Net::OpenSSH')) {
+            my (%opts,%runopts);
 
-            # child_fh set/not set
-            # $remote => Net::SSH settings
-            # remote host job accounting
-            my $sshpid = -1;
             my $host = $remote->{host};
-            my $user = $remote->{user};
-            my $port = $remote->{port};
-            if ($user) {
-                $host = $user . '@' . $host;
-            }
-            if ($port) {
-                $host .= ":$port";
-            }
-                
-
-            my ($in,$out,$err)=map Forks::Super::Job::Ipc::_gensym(),1..3;
-            $sshpid = Net::SSH::sshopen3(
-                $host, $in, $out, $err, @{$job->{cmd}});
-
-            # $sshpid is always valid? yes
-            $job->set_signal_pid($sshpid);
-            if ($job->{fh_config}{in}) {
-                print {$in} <STDIN>;
-            }
-            close $in;
-            my $z = CORE::waitpid $sshpid, 0;
-            my $c1 = $?;
-            print STDOUT <$out>;
-            print STDERR <$err>;
-            deinit_child();
-            debug("waitpid returned $z, exit code for $$ was $c1 ", $c1>>8)
-                if $job->{debug};
-            exit $c1 >> 8;
-        } elsif (CONFIG_module("Net::SSH::Perl")) {
-
-            open my $TTY,'>/dev/tty';
-
-            my ($user,$host) = split /\@/, $remote->{host};
-            if (!defined $host) {
-                $host = $user;
-                $user = $remote->{user};
-            }
-            if ($remote->{port}) {
-                $host .= ":" . $remote->{port};
-            }
-            my $ssh = Net::SSH::Perl->new($host);
-
-            if ($remote->{password}) {
-                my $login_ok = $ssh->login($user, $remote->{password}, 1);
-                print $TTY "login[1]: $login_ok\n";
-                if (!$login_ok) {
-                    # ...
-                }
-            } else {
-                my $login_ok = $ssh->login($user, "", 1);
-                print $TTY "login[2]: $login_ok\n";
-                if (!$login_ok) {
-                    # ...
-                }
+            foreach my $remoteopt (grep exists $remote->{$_},
+                                   qw(key_path user password passphrase
+                                      gateway ssh_cmd ctl_dir remote_shell
+                                      timeout strict_mode kill_ssh_on_timeout
+                                      async default_ssh_opts forward_agent
+                                      forward_X11 port)) {
+                $opts{$remoteopt} = $remote->{$remoteopt};
             }
 
-            my $cmd = "@{$job->{cmd}}";
             my $stdin;
             if ($job->{stdin}) {
-                $stdin = $job->{stdin};
+                if ('ARRAY' eq ref $job->{stdin}) {
+                    $stdin = join '', $job->{stdin}; # or join "\n",... ?
+                } else {
+                    $stdin = $job->{stdin};
+                }
             } elsif ($job->{fh_config}{in}) {
                 $stdin = join '', <STDIN>;
             }
 
+            untie *STDIN;
+            open STDIN,"</dev/null";
+            my $ssh = Net::OpenSSH->new($host,%opts);
+            croak $ssh->error if $ssh->error;
+            $job->set_signal_pid($ssh->get_master_pid);
+
+            if (defined $stdin) {
+                $runopts{stdin_data} = $stdin;
+            } else {
+                $runopts{stdin_discard} = 1;
+            }
+
+            my ($output,$errput) = $ssh->capture2(\%runopts, @cmd);
+            my $rc = $?;
+            print STDOUT $output;
+            print STDERR $errput;
+            $ssh->disconnect;
+            deinit_child();
+            exit $rc >> 8;
         } else {
-            croak "Alternate ssh support for remote option not implemented";
+            deinit_child();
+            carp "Alternate ssh support for remote option not implemented";
+            exit 66;
         }
         deinit_child();
         exit $? >> 8;
@@ -1733,25 +1710,72 @@ sub _preconfig_dependencies {
 
 sub _preconfig_remote {
     my $job = shift;
-    if (defined $job->{remote}) {
-        if (ref($job->{remote}) ne 'HASH') {
-            $job->{remote} = { host => $job->{remote} };
-        }
-        $job->{remote}{proto} ||= 'ssh';
-        $job->{remote}{__hosts__} = $job->{remote}{host};
-
-        if (!$job->{cmd}) {
-            if ($job->{exec}) {
-                $job->{cmd} = $job->{exec};
-            } else {
-                carp "fork: remote => ... specified without  cmd => ... ! ",
-                     "remote spec will be ignored";
-                $job->{remote_disabled} = delete $job->{remote};
-            }
+    return if !defined $job->{remote};
+    my $remote = $job->{remote};
+    if (!$job->{cmd}) {
+        if ($job->{exec}) {
+            $job->{cmd} = $job->{exec};
+        } else {
+            carp "fork: remote => ... specified without  cmd => ... ! ",
+            "remote spec will be ignored";
+            $job->{remote_disabled} = delete $job->{remote};
+            return;
         }
     }
-    if (defined $job->{rhost}) {
-        croak "Use  remote => ...  not  rhost => ...";
+
+    if ('ARRAY' ne ref $job->{remote}) {
+        # prior to launch, ensure that $job->{remote} is an array reference
+        # of allowable remote specs.
+        #
+        # after launch, $job->{remote} will point to the actual remote spec
+        # that was used to launch the job (see _can_launch_remote)
+        $job->{remote} = [ $job->{remote} ];
+    }
+    $job->{remote} = [
+        map {
+            if (!ref($_)) {
+                { host => $_ };
+            } elsif (!defined($_->{host})) {
+                ()
+            } elsif (defined($_->{proto}) && $_->{proto} ne 'ssh') {
+                carp "Forks::Super: Only 'ssh' protocol supported in remote";
+                ()
+            } elsif (ref($_->{host}) eq 'ARRAY') {
+                # not documented, but  remote => { host=>[host1,host2],user=>... }
+                # is supported
+                my $spec = $_;
+                map {
+                    my $spec2 = { %$spec };
+                    $spec2->{host} = $_;
+                    $spec2;
+                } @{$_->{host}};
+            } else {
+                $_
+            }
+        } @{$job->{remote}}
+        ];
+    if (@{$job->{remote}} == 0) {
+        $job->{__error} = "No valid remote specs in remote param $remote";
+        return;
+    }
+    foreach my $spec (@{$job->{remote}}) {
+        my $host = $spec->{host};
+        my ($up,$hp) = split /(?<!\\)@/, $host, 2;
+        if ($up && $hp) {
+            my ($u,$p) = split /(?<!\\):/, $up, 2;
+            if ($u && $p) {
+                $spec->{user} ||= $u;
+                $spec->{password} ||= $p;
+            } else {
+                $spec->{user} ||= $up;
+            }
+            $host = $hp;
+        }
+        my ($h,$p) = split /(?<!\\):/, $host, 2;
+        if ($h && $p) {
+            $spec->{host} = $h;
+            $spec->{port} ||= $p;
+        }
     }
     return;
 }
@@ -1878,45 +1902,6 @@ sub _config_dir {
     return;
 }
 
-sub _INSIDE_END_QUEUE {
-
-  # for RT#76868
-  if ($INC{"Devel/GlobalDestruction.pm"}) {
-    return in_global_destruction();
-  } else {
-    return $INSIDE_END_QUEUE;
-  }
-}
-
-END {
-    no warnings 'internal';
-    $INSIDE_END_QUEUE = 1;
-    if ($$ == ($Forks::Super::MAIN_PID ||= $$)) {
-
-	# disable SIGCHLD handler during cleanup. Hopefully this will fix
-	# intermittent test failures where all subtests pass but the
-	# test exits with non-zero exit status (e.g., t/42d-filehandles.t)
-
-	untie %SIG;
-	eval {
-	    no warnings;                    ## no critic (NoWarnings)
-	    $SIG{CHLD} = undef;
-	    1;
-	} or do {
-	    $SIG{CHLD} = 'IGNORE';
-	};
-
-	Forks::Super::Queue::_cleanup();
-	Forks::Super::Job::Ipc::_cleanup();
-    } else {
-        if (defined($this) && defined($this->{_cleanup_code})) {
-            no strict 'refs';
-            $this->{_cleanup_code}->();
-        }
-	Forks::Super::Job::Timeout::_cleanup_child();
-    }
-}
-
 sub acquire {
     my ($job, $n, $timeout) = @_;
     $job = &this if $job eq __PACKAGE__;
@@ -1940,6 +1925,53 @@ sub acquireAndRelease {
     return $job->acquire($n,$timeout) && $job->release($n);
 }
 
+######################################################################
+
+# Global destruction handling. There is a lot of cleanup in this
+# distribution and a lot of functions need to know if they are
+# being called during global destruction.
+
+BEGIN {
+    if (defined ${^GLOBAL_PHASE}) {  # Perl v>=5.14.0
+        eval 'sub _INSIDE_END_QUEUE () {
+                  ${^GLOBAL_PHASE} eq q{DESTRUCT} &&  __END() }; 1';
+    } else {
+        require B;
+        eval 'sub _INSIDE_END_QUEUE () { ${B::main_cv()} == 0 && __END() }; 1';
+    }
+}
+
+END { &__END }
+
+sub __END {
+    no warnings 'internal', 'redefine';
+    *_INSIDE_END_QUEUE = sub () { 1 };
+
+    if ($$ == ($Forks::Super::MAIN_PID ||= $$)) {
+
+	# disable SIGCHLD handler during cleanup. Hopefully this will fix
+	# intermittent test failures where all subtests pass but the
+	# test exits with non-zero exit status (e.g., t/42d-filehandles.t)
+
+	untie %SIG;
+	eval {
+	    no warnings;                    ## no critic (NoWarnings)
+	    $SIG{CHLD} = undef; 1;
+	} or do {
+	    $SIG{CHLD} = 'IGNORE';
+	};
+	Forks::Super::Queue::_cleanup();
+	Forks::Super::Job::Ipc::_cleanup();
+    } else {
+        if (defined($this) && defined($this->{_cleanup_code})) {
+            no strict 'refs';
+            $this->{_cleanup_code}->();
+        }
+	Forks::Super::Job::Timeout::_cleanup_child();
+    }
+    1;
+}
+
 #############################################################################
 # Package methods (meant to be called as Forks::Super::Job::xxx(@args))
 
@@ -2199,12 +2231,14 @@ sub count_processes {
 
 sub count_active_processes_on_host {
     my $host = shift;
-    return scalar grep { 
-        $_->{remote} && $_->{remote}{host} &&
-            $_->{remote}{host} eq $host &&
-            defined($_->{state})
-            && $_->{state} eq 'ACTIVE'
+    my $n = scalar grep {
+        $_->{remote} &&
+            defined($_->{state}) &&
+            $_->{state} eq 'ACTIVE' &&
+            $_->{remote}{host} &&
+            $_->{remote}{host} eq $host
     } @ALL_JOBS;
+    return $n;
 }
 
 sub init_child {
@@ -2327,7 +2361,7 @@ Forks::Super::Job - object representing a background task
 
 =head1 VERSION
 
-0.85
+0.86
 
 =head1 SYNOPSIS
 
