@@ -33,7 +33,7 @@ $| = 1;
 
 our @ISA = qw(Exporter);
 our @EXPORT = qw(close_fh);
-our $VERSION = '0.85';
+our $VERSION = '0.86';
 our $NO_README = 0;
 
 our (%FILENO, %SIG_OLD, $IPC_COUNT, $IPC_DIR_DEDICATED,
@@ -179,7 +179,8 @@ sub _safeopen ($*$$;%) {
                      "handles");
             }
         }
-        return if 'rescue' ne $Forks::Super::ON_TOO_MANY_OPEN_FILEHANDLES;
+        return if !$Forks::Super::ON_TOO_MANY_OPEN_FILEHANDLES ||
+            'rescue' ne $Forks::Super::ON_TOO_MANY_OPEN_FILEHANDLES;
 
 	Forks::Super::try_to_close_some_open_filehandles();
     }
@@ -1311,7 +1312,7 @@ sub _END_foreground_cleanup {
 		    values %Forks::Super::CHILD_STDERR) {
 	# _close($fh);
 	delete $__OPEN_FH{fileno($fh) || -1};
-	$__OPEN_FH -= close $fh;
+	$__OPEN_FH -= close($fh) || 0;
     }
 
     # daemonize if there is anything to clean up
@@ -1572,6 +1573,7 @@ sub __config_fh_parent_stdin_pipes {
     my $job = shift;
     my $fh_config = $job->{fh_config};
     if ($USE_TIE_PH) {
+        no warnings 'io';
 	my $ph = _gensym();
 	tie *$ph, 'Forks::Super::Tie::IPCPipeHandle',
                 $fh_config->{p_to_in}, $ph;
@@ -1682,6 +1684,7 @@ sub __config_fh_parent_stdout_pipes {
     my $fh_config = $job->{fh_config};
 
     if ($USE_TIE_PH) {
+        no warnings 'io';
 	my $ph = _gensym();
 	tie *$ph, 'Forks::Super::Tie::IPCPipeHandle', $fh_config->{p_out}, $ph;
 	$job->{child_stdout}
@@ -1815,6 +1818,8 @@ sub __config_fh_parent_stderr_pipes {
     my $fh_config = $job->{fh_config};
 
     if ($USE_TIE_PH) {
+        no warnings 'io';
+        
 	my $ph = _gensym();
 	tie *$ph, 'Forks::Super::Tie::IPCPipeHandle', $fh_config->{p_err}, $ph;
 
@@ -2011,6 +2016,7 @@ sub __config_fh_child_stdin_sockets {
 
 sub __config_fh_child_stdin_pipes {
     my $job = shift;
+    no warnings 'io';
     my $fh_config = $job->{fh_config};
     push @{$job->{child_fh_close}}, $fh_config->{p_in};
     close STDIN;
@@ -2092,6 +2098,7 @@ sub _config_fh_child_stdout {
 
 sub __config_fh_child_stdout_sockets {
     my $job = shift;
+    no warnings 'io';
     my $fh_config = $job->{fh_config};
 
     close STDOUT;
@@ -2128,8 +2135,8 @@ sub __config_fh_child_stdout_sockets {
 
 sub __config_fh_child_stdout_pipes {
     my $job = shift;
+    no warnings 'io';
     my $fh_config = $job->{fh_config};
-
     close STDOUT;
     if ($USE_TIE_PH) {
 	my $fh = _gensym();
@@ -2261,6 +2268,7 @@ sub __config_fh_child_stderr_sockets {
 
 sub __config_fh_child_stderr_pipes {
     my $job = shift;
+    no warnings 'io';
     my $fh_config = $job->{fh_config};
 
     push @{$job->{child_fh_close}}, $fh_config->{p_to_err};
@@ -3372,7 +3380,7 @@ Forks::Super::Job::Ipc - interprocess communication routines for Forks::Super
 
 =head1 VERSION
 
-0.85
+0.86
 
 =head1 DESCRIPTION
 
