@@ -59,7 +59,7 @@ use overload # XXX - what is overloading for? is it necessary?
 		         : atan2($_[0]->_fetch, $_[1]) }
 ;
 
-our $VERSION = '0.85';
+our $VERSION = '0.86';
 
 # "protocols" for serializing data and the methods used
 # to carry out the serialization
@@ -200,7 +200,7 @@ sub _fetch {
 
     if (!$self->{value_set}) {
 	if (!$self->{job}->is_complete) {
-            return if $Forks::Super::Job::INSIDE_END_QUEUE;
+            return if &Forks::Super::Job::_INSIDE_END_QUEUE;
 	    my $pid = Forks::Super::waitpid $self->{job_id}, WREAP_BG_OK;
             no warnings 'numeric';
 	    if ($pid != $self->{job}{real_pid} && $pid != $self->{job}{pid}) {
@@ -264,7 +264,7 @@ Forks::Super::LazyEval::BackgroundScalar
 
 =head1 VERSION
 
-0.85
+0.86
 
 =head1 DESCRIPTION
 
