@@ -24,7 +24,7 @@ use IO::Pipe;
 use IO::Handle;
 
 our @ISA = qw(IO::Pipe IO::Handle);
-our $VERSION = '0.85';
+our $VERSION = '0.86';
 
 sub TIEHANDLE {
     my ($class, $real_pipe, $glob) = @_;
@@ -209,6 +209,14 @@ sub CLOSE {
     return;
 }
 
+sub UNTIE {
+    my $self = shift;
+    if ($self->{PIPE}) {
+        close $self->{PIPE};
+    }
+    return;
+}
+
 sub DESTROY {
     my $self = shift;
     $self = {};
