@@ -673,7 +673,9 @@ sub _summarize_failures {
 	    print "\t$test_file#$test_no ";
 	    if ($fail{$test_file}{$test_no} > 1) {
 		print "$fail{$test_file}{$test_no} times\n";
-	    }
+	    } else {
+                print "\n";
+            }
 	}
     }
     print "================\n";
@@ -824,7 +826,7 @@ forked_harness.pl - run tests in parallel with Forks::Super
 
 =head1 VERSION
 
-0.85
+0.86
 
 =head1 SYNOPSIS
 
